import pandas as pd
import json
from dku_plugin_config_loading import DkuConfigLoadingIsochrone
import utils
from dkuconstants import RECIPE
import dataiku

import logging

logging.basicConfig(level="INFO")

def parse_one_row(row):
    if row:
        return pd.Series(json.loads(row))
    else:
        return pd.Series({"isochrone": None})

dku_license_id = dataiku.api_client().get_instance_info().raw["licenseId"]

dku_config = DkuConfigLoadingIsochrone()
settings = dku_config.load_settings()
input_dataframe = settings.input_dataset.get_dataframe(infer_with_pandas=False)
geo_router_class = utils.get_api_geo_router(RECIPE.ROUTE)
geo_router = geo_router_class(api_endpoint_url=settings.api_endpoint_url, dku_license_id=dku_license_id)
transport_mode = (
    input_dataframe[settings.transport_mode_column].dropna().unique()
    if settings.transport_mode == "transport_mode_column"
    else [settings.transport_mode]
)
geo_router.process_df_for_isochrones(
    input_dataset=settings.input_dataset,
    output_dataset=settings.output_dataset,
    stream_chunk_size=settings.stream_chunk_size,
    coords_col=settings.coords_column,
    time_threshold=settings.time_threshold,
    transport_mode=settings.transport_mode,
    transport_mode_col=settings.transport_mode_column,
    postprocess_func=utils.postprocess_df,
    postprocess_func_args=[parse_one_row]
)
