import pandas as pd
import json
from dku_plugin_config_loading import DkuConfigLoadingRouting
import utils
from dkuconstants import RECIPE
from dkulib.io_utils.dku_utils import set_column_descriptions
import logging
import dataiku

logging.basicConfig(level="INFO")

def parse_one_row(row, get_itinerary):
    if row:
        response = json.loads(row)
        if get_itinerary:
            return pd.Series({"time": response.get("time"),
                              "distance": response.get("distance"),
                              "itinerary": response.get("itinerary")})
        else:
            return pd.Series({"time": response.get("time"),
                              "distance": response.get("distance")})
    else:
        if get_itinerary:
            return pd.Series({"time": None, "distance": None, "itinerary": None})
        else:
            return pd.Series({"time": None, "distance": None})

dku_license_id = dataiku.api_client().get_instance_info().raw["licenseId"]

dku_config = DkuConfigLoadingRouting()
settings = dku_config.load_settings()
input_dataframe = settings.input_dataset.get_dataframe(infer_with_pandas=False)
geo_router_class = utils.get_api_geo_router(RECIPE.ROUTE)
geo_router = geo_router_class(api_endpoint_url=settings.api_endpoint_url, dku_license_id=dku_license_id)
transport_mode = (
    input_dataframe[settings.transport_mode_column].dropna().unique()
    if settings.transport_mode == "transport_mode_column"
    else [settings.transport_mode]
)

geo_router.process_df_for_routes(
    input_dataset=settings.input_dataset,
    from_col=settings.from_column,
    to_col=settings.to_column,
    transport_mode=settings.transport_mode,
    distance_unit=settings.distance_unit,
    output_dataset=settings.output_dataset,
    stream_chunk_size=settings.stream_chunk_size,
    get_itinerary=settings.get_itinerary,
    transport_mode_col=settings.transport_mode_column,
    postprocess_func=utils.postprocess_df,
    postprocess_func_args=[parse_one_row, settings.get_itinerary]
)

set_column_descriptions(
    output_dataset=settings.output_dataset,
    column_descriptions={"time": "Time in minutes", "distance": f"Distance in {settings.distance_unit.value}"},
)
