from geo_router import GeoRouter
from dkuconstants import TRANSPORT_MODE
from dkuconstants import DISTANCE_UNIT
from dkuconstants import CONVERSION

class GraphhopperGeoRouter(GeoRouter):
    def __init__(self, api_endpoint_url, dku_license_id=None):
        super(GraphhopperGeoRouter, self).__init__(api_endpoint_url, dku_license_id)
        self.route_endpoint = "route"
        self.isochrone_endpoint = "isochrone"
        self.query_method = "GET"

    def enrich_headers(self, headers):
        super(GraphhopperGeoRouter, self).enrich_headers(headers)
        headers["X-Prefered-Engine"] = "Graphhopper"
        headers["X-Api-Version"] = "None"

    def get_local_transport_mode(self, global_transport_mode):
        if global_transport_mode == TRANSPORT_MODE.CAR:
            return "car"
        elif global_transport_mode == TRANSPORT_MODE.BIKE:
            return "bike"
        elif global_transport_mode == TRANSPORT_MODE.PEDESTRIAN:
            return "foot"
        else:
            raise ValueError(f"Invalid transport mode: {global_transport_mode}. Supported transport modes are : car, bicycle, pedestrian")

    def get_isochrone_querystring(self, coords, time_threshold, global_transport_mode, **kwargs):
        return {
            "point": f"{coords[1]},{coords[0]}",
            "time_limit": int(time_threshold * 60),
            "profile": self.get_local_transport_mode(global_transport_mode),
        }

    def get_route_querystring(self, from_coords, to_coords, global_transport_mode, get_itinerary, **kwargs):
        return {
            "point": [f"{from_coords[1]},{from_coords[0]}", f"{to_coords[1]},{to_coords[0]}"],
            "profile": self.get_local_transport_mode(global_transport_mode),
            "points_encoded": True,
            "calc_points": get_itinerary,
            "instructions" : False
        }

    def get_isochrone_geometry_from_response(self, response):
        return response.get("polygons")

    def get_itinerary_from_response(self, response):
        itinerary = self.decode_encoded_polyline(response["paths"][0]["points"])
        if len(itinerary) <=1:
            return {"type":"LineString", "coordinates": []}
        return {"type": "LineString", "coordinates": self.decode_encoded_polyline(response["paths"][0]["points"])}

    def get_time_from_response(self, response):
        time_in_ms = response["paths"][0]["time"]
        return round(time_in_ms / CONVERSION.MINUTE_IN_MILLISECONDS.value)

    def get_distance_from_response(self, response, distance_unit):
        distance_in_meters = response["paths"][0]["distance"]
        if distance_unit == DISTANCE_UNIT.METERS:
            return distance_in_meters
        elif distance_unit == DISTANCE_UNIT.MILES:
            return distance_in_meters / CONVERSION.MILE_IN_METERS.value
        elif distance_unit == DISTANCE_UNIT.KILOMETERS:
            return distance_in_meters / CONVERSION.KILOMETER_IN_METERS.value
